#include "esp_mqtt.h"

#include <stdio.h>
#include <stdint.h>
#include <stddef.h>
#include <string.h>
#include "esp_wifi.h"
#include "esp_system.h"
#include "nvs_flash.h"
#include "esp_event_loop.h"
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/semphr.h"
#include "freertos/queue.h"
#include "freertos/event_groups.h"
#include "lwip/sockets.h"
#include "lwip/dns.h"
#include "lwip/netdb.h"
#include "cJSON.h"
#include "esp_log.h"
#include "mqtt_client.h"

#include "grill_data.h"
#include "esp_config.h"


#define TAG                 "MQTT"
#define MqttRecvMaxBuf		700


char *getFloatString(float f);
char *getIntString(int i);
char *getHexString(unsigned char hex);

static const char *lastmsg = "OffLine";
char retainTopic[64];
char phoneTopic[64];			//
char devTopic[64];				//
char mqttIP[64];
//是否订阅过主题，如果订阅过了下载订阅的时候先清空再订阅
char phoneTopicID = 0;
esp_mqtt_client_handle_t key_client = NULL; 
unsigned char mqttSendBuf[MqttRecvMaxBuf];
//当前BLE发送的长度
unsigned short mqttSendBufLen = 0;

void subscribeAll(esp_mqtt_client_handle_t client){
	//这里订阅主题	
	if(phoneTopicID!=0){
		esp_mqtt_client_unsubscribe(client,phoneTopic);
	}
	phoneTopicID = esp_mqtt_client_subscribe(client, phoneTopic, 0);
	ESP_LOGI(TAG, "sent subscribe successful:%s , msg_id=%d", phoneTopic,phoneTopicID);
}
//发布消息到手机
char publicToPhoneMsg(char *msg,unsigned int msglen){
	if(key_client!=NULL){
		if(esp_mqtt_client_publish(key_client, devTopic, msg, msglen, 0, 0)>=0){
			return 1;
		}
	}	
	return 0;
}
void dealFromePhoneMsg(char *topic,char *msg,unsigned int msglen){	
	memset(mqttSendBuf,0,MqttRecvMaxBuf);
	mqttSendBufLen = 0;
	if(strcmp(topic,phoneTopic)==0){
		//memset(bleSendBuf,0,BleRecvMaxBuf);
		dealGrillData((unsigned char *)msg,msglen,mqttSendBuf,&mqttSendBufLen);
		if(mqttSendBufLen>0){
			publicToPhoneMsg((char *)mqttSendBuf,mqttSendBufLen);
		}
	}else{
		ESP_LOGI(TAG,"unkonw topic:%s",topic);
	}
}
static esp_err_t mqtt_event_handler(esp_mqtt_event_handle_t event){
    esp_mqtt_client_handle_t client = event->client;
    int msg_id;
    // your_context_t *context = event->context;
    switch (event->event_id) {
		case MQTT_EVENT_BEFORE_CONNECT:
			break;
        case MQTT_EVENT_CONNECTED:{
            ESP_LOGI(TAG, "MQTT_EVENT_CONNECTED");
			key_client = client;
			espInfo.isMqttStart = 3;
			subscribeAll(client);
		}
		break;
        case MQTT_EVENT_DISCONNECTED:
            ESP_LOGI(TAG, "MQTT_EVENT_DISCONNECTED");
			espInfo.isMqttStart = 4;
            break;
        case MQTT_EVENT_SUBSCRIBED:
            ESP_LOGI(TAG, "MQTT_EVENT_SUBSCRIBED, msg_id=%d", event->msg_id);
            //msg_id = esp_mqtt_client_publish(client, "/topic/qos0", "data", 0, 0, 0);
            //ESP_LOGI(TAG, "sent publish successful, msg_id=%d", msg_id);
            break;
        case MQTT_EVENT_UNSUBSCRIBED:
            ESP_LOGI(TAG, "MQTT_EVENT_UNSUBSCRIBED, msg_id=%d", event->msg_id);
            break;
        case MQTT_EVENT_PUBLISHED:
            ESP_LOGI(TAG, "MQTT_EVENT_PUBLISHED, msg_id=%d", event->msg_id);
            break;
        case MQTT_EVENT_DATA:
            // ESP_LOGI(TAG, "MQTT_EVENT_DATA");
            // printf("TOPIC=%s   len:%d\r\n",event->topic,event->data_len);
            // printf("DATA=%.*s\r\n", event->data_len, event->data);
		{
			
			char xtopic[64];
			char xdata[512];
			memset((unsigned char *)xtopic,0,64);
			memset((unsigned char *)xdata,0,512);
			memcpy(xtopic,event->topic,event->topic_len );
			memcpy(xdata,event->data,event->data_len);

			dealFromePhoneMsg(xtopic,xdata,event->data_len);
		}
		break;
        case MQTT_EVENT_ERROR:
            ESP_LOGI(TAG, "MQTT_EVENT_ERROR");
            break;
		default:
			break;
    }
    return ESP_OK;
}
//关闭MQTT，重启的时候才会使用
void stopMqtt(void){
	esp_mqtt_client_stop(key_client);
}

void initMqtt(char *mqttip,int port,char *user,char *pass,char *clientID){
	memset(retainTopic,0,64);
	memset(phoneTopic,0,64);
	memset(devTopic,0,64);
	memset(mqttIP,0,64);
	
	phoneTopicID = 0;
	sprintf(retainTopic,"et/will/%s/retain",espInfo.strdevid);
	sprintf(phoneTopic,"et/%s/msgToDev",espInfo.strdevid);
	sprintf(devTopic,"et/%s/msgToPhone",espInfo.strdevid);
//	sprintf(mqttIP,"%s:%d",mqttip,port);
	//这里需要增加一个域名翻译功能，找到域名对应的IP
    ip_addr_t target_addr;
    struct addrinfo hint;
    struct addrinfo *res = NULL;
    memset(&hint, 0, sizeof(hint));
    memset(&target_addr, 0, sizeof(target_addr));
    getaddrinfo(mqttip, NULL, &hint, &res);
    struct in_addr addr4 = ((struct sockaddr_in *) (res->ai_addr))->sin_addr;
    inet_addr_to_ip4addr(ip_2_ip4(&target_addr), &addr4);
    freeaddrinfo(res);
	sprintf(mqttIP,"%d.%d.%d.%d",addr4.s_addr&0xff,(addr4.s_addr>>8)&0xff,(addr4.s_addr>>16)&0xff,(addr4.s_addr>>24)&0xff);	
	esp_mqtt_client_config_t mqtt_cfg = {
        //.uri = CONFIG_BROKER_URL,
        .host = "129.28.190.20",
		.port = 1883,
		.event_handle = mqtt_event_handler,
		.task_stack = 10240,
		//.client_id = mqttid
        // .user_context = (void *)your_context
    };
	mqtt_cfg.host = (const char *)mqttIP;
	// mqtt_cfg.host =  (const char *)"zhouhai.allhex.com";
	mqtt_cfg.username = user;
	mqtt_cfg.password = pass;
	mqtt_cfg.client_id = clientID;
	mqtt_cfg.keepalive = 15;
    esp_mqtt_client_handle_t client = esp_mqtt_client_init(&mqtt_cfg);
    esp_mqtt_client_start(client);
	ESP_LOGI(TAG,"打开了MQTT:%s  %s",mqttip,mqttIP);
}


char xfstr[32];
char *getFloatString(float f){
	//float xf = (unsigned int)(f*100)/100.0;
	//printf("get float:%lf %lf \r\n",xf,f);
	//memset(xfstr,0,32);
	//sprintf(xfstr,"%lf",xf);
	sprintf(xfstr,"220.12");
	return xfstr;
}

char istr[32];
char *getIntString(int i){
	memset(istr,0,32);
	sprintf(istr,"%d",i);
	return istr;
}

char *getUintString(unsigned int i){
	memset(istr,0,32);
	sprintf(istr,"%d",i);
	return istr;
}


char hstr[32];
char *getHexString(unsigned char hex){
	memset(hstr,0,32);
	sprintf(hstr,"%02X",hex);
	return hstr;
}


