#include "esp_new_wifi.h"

#include "string.h"
#include "stdio.h"
#include "esp_wifi.h"
#include "esp_event_loop.h"
#include "esp_log.h"
#include "esp_system.h"
#include "freertos/event_groups.h"
#include "esp_wifi.h"
#include "esp_event_loop.h"
#include "esp_log.h"
#include "esp_http_client.h"

#include "public_method.h"

#include "esp_mqtt.h"
#include "esp_config.h"

#define TAG 				"IotWiFi"

void initWiFi(void);
char regEspDev(void);
char httpGetLinuxTime(char *devid,unsigned long *time);


const int CONNECTED_BIT =       BIT0;
const int SCAN_DONE_BIT =       BIT1;
const int WiFiStart_BIT =       BIT2;
const int ESPTOUCH_DONE_BIT =   BIT3;
const int DISCONNECT_BIT =      BIT4;
static EventGroupHandle_t wifi_event_group;
//wifi是否连接ok了
unsigned char isWiFiConnected = 0;

static void elle_iot_wifi_task(void *arg){
    char wifiStatus = 0;
    ESP_LOGI(TAG,"WiFi thread start");
    loadWiFiConfig();
connecting:
    while(1){
        if(strlen(espInfo.wifi_ssid)>0){
			ESP_LOGI(TAG,"连接WiFi:%s 密码:%s",espInfo.wifi_ssid,espInfo.wifi_pass);
            if(connectWiFi(espInfo.wifi_ssid,espInfo.wifi_pass)>0){
                ESP_LOGI(TAG,"connect OK and jump to configed");
                espInfo.wifiStatus = 3;
                goto connected;
            }else{
                ESP_LOGI(TAG,"connect failed and jump to status");
                vTaskDelay(5000 / portTICK_RATE_MS);
            }            
		}
        vTaskDelay(1000 / portTICK_RATE_MS);
    }

connected:
    while(1){

        if(espInfo.wifiStatus!=3){
            disConnectedWiFi();   
            vTaskDelay(2000 / portTICK_RATE_MS);
            goto connecting;
        }     

        if(httpGetLinuxTime(espInfo.strdevid,&(espInfo.worldTime))>0){	            
        }else{
            vTaskDelay(3000 / portTICK_RATE_MS);
            ESP_LOGI(TAG,"获取linux时间失败");
            continue;
        }
        //进行注册 - 如果没有MQTT信息的话
        if(espInfo.isMqttStart<1){
            vTaskDelay(1000 / portTICK_RATE_MS);
            if(regEspDev()>0){  
                ESP_LOGI(TAG,"注册设备成功 也把WiFi保存下来");
                saveWiFiSsidPass(espInfo.wifi_ssid,espInfo.wifi_pass);
                loadMqttInfo();
            }else{
                ESP_LOGI(TAG,"注册设备失败");
                vTaskDelay(3000 / portTICK_RATE_MS);
                goto connected;
            }
        }
        //启动MQTT 已经进行云端mqtt注册
        if(espInfo.isMqttStart == 1){            
            espInfo.isMqttStart = 2;
            initMqtt(espInfo.mqttIp,atoi(espInfo.mqttPort),espInfo.mqttUser,espInfo.mqttPass,espInfo.productKey);
        }else{
            ESP_LOGI(TAG,"mqtt don't need init");
        } 
        // //查询是否需要升级，由客户决定什么时候升级
        // if(startUpgrade()>0){
        //     //升级成功了 - 自动重启
        //     //可以的话，提交下升级成功的log
        //     esp_restart();
        // }

        //维持状态 - WiFi在线状态
        while(1){
            if(wifiStatus!=espInfo.wifiStatus){
                wifiStatus = espInfo.wifiStatus;
                if(wifiStatus==4){
                    //密码错误了，准备3s后重新连接
                    vTaskDelay(3000 / portTICK_RATE_MS);
                    ESP_LOGI(TAG,"WiFi掉线了，进行重连");
                    goto connecting;
                }
                if(wifiStatus==5){
                    ESP_LOGI(TAG,"如果客户更改了SSID，则在这里进行再次连接");
                    //关闭MQTT
                    stopMqtt();
                    vTaskDelay(1500 / portTICK_RATE_MS);
                    goto connecting;
                }
            }
            vTaskDelay(1000 / portTICK_RATE_MS);
            ESP_LOGI(TAG,"WiFiStatus:%d   RAM:%d   Time:%s  ",wifiStatus,esp_get_free_heap_size(),getWorldTimeStr());
        }             
        vTaskDelay(1000 / portTICK_RATE_MS);
    }


}

void initEspNewWiFi(void){
    saveWiFiSsidPass("SmartHome-Next","4321qq4321");
    initWiFi();
    xTaskCreatePinnedToCore(elle_iot_wifi_task, "ElleWiFiTask", 8192, NULL, 0, NULL, 0);
}



static void event_handler(void* arg, esp_event_base_t event_base,int32_t event_id, void* event_data)
{
    if (event_base == WIFI_EVENT && event_id == WIFI_EVENT_STA_START) {

    } else if (event_base == WIFI_EVENT && event_id == WIFI_EVENT_STA_DISCONNECTED) {
        wifi_event_sta_disconnected_t *disconnected = (wifi_event_sta_disconnected_t*) event_data;
        ESP_LOGE(TAG, "Disconnect reason : %d", disconnected->reason);     
        xEventGroupSetBits(wifi_event_group, DISCONNECT_BIT);
        ESP_LOGI(TAG,"connect to the AP fail");
        isWiFiConnected = 0;
    } else if( event_base == WIFI_EVENT && event_id == WIFI_EVENT_STA_CONNECTED ){
        ESP_LOGI(TAG,"connect to the AP succeed");
    } else if (event_base == IP_EVENT && event_id == IP_EVENT_STA_GOT_IP) {
        ip_event_got_ip_t* event = (ip_event_got_ip_t*) event_data;
        ESP_LOGI(TAG, "got ip:" IPSTR, IP2STR(&event->ip_info.ip));
        xEventGroupSetBits(wifi_event_group, CONNECTED_BIT);
        isWiFiConnected = 1;
    }
}

void initWiFi(void){
    wifi_event_group = xEventGroupCreate();
    ESP_ERROR_CHECK(esp_netif_init());
    ESP_ERROR_CHECK(esp_event_loop_create_default());
    esp_netif_create_default_wifi_sta();
    wifi_init_config_t cfg = WIFI_INIT_CONFIG_DEFAULT();
    ESP_ERROR_CHECK(esp_wifi_init(&cfg));
    esp_event_handler_instance_t instance_any_id;
    esp_event_handler_instance_t instance_got_ip;
    ESP_ERROR_CHECK(esp_event_handler_instance_register(WIFI_EVENT,
                                                        ESP_EVENT_ANY_ID,
                                                        &event_handler,
                                                        NULL,
                                                        &instance_any_id));
    ESP_ERROR_CHECK(esp_event_handler_instance_register(IP_EVENT,
                                                        IP_EVENT_STA_GOT_IP,
                                                        &event_handler,
                                                        NULL,
                                                        &instance_got_ip));
    ESP_ERROR_CHECK(esp_wifi_set_mode(WIFI_MODE_STA) );
    ESP_ERROR_CHECK(esp_wifi_start());
    esp_wifi_set_bandwidth(ESP_IF_WIFI_STA,WIFI_BW_HT20);
	
}

char connectWiFi(char *ssid,char *pwd){
	wifi_config_t wifi_config={
	    .sta = {
	        .ssid = "",
	        .password = "",
	    },
	};
	if(ssid!=NULL){
		memcpy(wifi_config.sta.ssid,ssid,strlen(ssid));
		if(pwd!=NULL)
		{
		 	memcpy(wifi_config.sta.password,pwd,strlen(pwd));
		}
	}

    ESP_LOGI(TAG, "Setting WiFi configuration SSID %s Pwd %s ", wifi_config.sta.ssid,wifi_config.sta.password);
    ESP_ERROR_CHECK(esp_wifi_set_mode(WIFI_MODE_STA));
    ESP_ERROR_CHECK(esp_wifi_set_config(ESP_IF_WIFI_STA, &wifi_config));
    ESP_ERROR_CHECK(esp_wifi_start());
	esp_wifi_connect();
    EventBits_t connected_bit = xEventGroupWaitBits(wifi_event_group, CONNECTED_BIT|DISCONNECT_BIT, true, false, (15)*1000 / portTICK_RATE_MS);
	if (connected_bit & CONNECTED_BIT) {
		return 1;
	}
	return 0;
}

void disConnectedWiFi(void){
    ESP_LOGI(TAG,"主动断开WiFi");
    esp_wifi_disconnect();
}





#define UrlMaxLen					256
#define MAX_HTTP_RECV_BUFFER 		1224
#define MAX_RECV_BUFFER 			1224
#define MAX_HTTP_SEND_BUFFER 		512
#define DevServer	"http://xapi.elletechnology.com/DevPublicService"

char content[MAX_HTTP_RECV_BUFFER];
int contentLen = 0;

esp_err_t _http_event_handler(esp_http_client_event_t *evt)
{
    switch(evt->event_id) {
        case HTTP_EVENT_ERROR:
            ESP_LOGD(TAG, "HTTP_EVENT_ERROR");
            break;
        case HTTP_EVENT_ON_CONNECTED:
            ESP_LOGD(TAG, "HTTP_EVENT_ON_CONNECTED");
			memset(content,0,MAX_HTTP_RECV_BUFFER);
			contentLen = 0;
            break;
        case HTTP_EVENT_HEADER_SENT:
            ESP_LOGD(TAG, "HTTP_EVENT_HEADER_SENT");
            break;
        case HTTP_EVENT_ON_HEADER:
            ESP_LOGD(TAG, "HTTP_EVENT_ON_HEADER, key=%s, value=%s", evt->header_key, evt->header_value);
            break;
        case HTTP_EVENT_ON_DATA:
            ESP_LOGD(TAG, "HTTP_EVENT_ON_DATA, len=%d", evt->data_len);
			
			memcpy(&content[contentLen],evt->data,evt->data_len);
			contentLen += evt->data_len;
			
            if (!esp_http_client_is_chunked_response(evt->client)) {
                // Write out data
                // printf("%.*s", evt->data_len, (char*)evt->data);
            }
            break;
        case HTTP_EVENT_ON_FINISH:
            ESP_LOGD(TAG, "HTTP_EVENT_ON_FINISH");
            break;
        case HTTP_EVENT_DISCONNECTED:
            ESP_LOGD(TAG, "HTTP_EVENT_DISCONNECTED");
            break;
    }
    return ESP_OK;
}

char httpGetLinuxTime(char *devid,unsigned long *time){
	char url[UrlMaxLen];
	int code = 0;
	char content[MAX_HTTP_RECV_BUFFER];
	int contentLen = 0;
	int read_len = 0;
	
	memset(url,0,UrlMaxLen);
	memset(content,0,MAX_HTTP_RECV_BUFFER);
	
	//sprintf(url,"%s/dev/getsystime?mac=%s ",DevServer,devid);
	sprintf(url,"%s/dev/getsystime",DevServer);
	ESP_LOGI(TAG,"ReportMsgUrl:%s  ",url);
	
	esp_http_client_config_t config = {
        .url = "http://www.elletechnology.com/DevPublicService/dev/getsystime?mac=84F3EB895D0E",
		.event_handler = _http_event_handler,
		.timeout_ms = 20000,
    };
	
    esp_http_client_handle_t client = esp_http_client_init(&config);
    
    esp_http_client_set_method(client, HTTP_METHOD_GET);
	esp_http_client_set_url(client, url);
	esp_http_client_set_header(client,"mac",devid);

	esp_err_t err = esp_http_client_perform(client);
    if (err == ESP_OK) {
		contentLen = esp_http_client_get_content_length(client);
		code = esp_http_client_get_status_code(client);
		ESP_LOGI(TAG,"Code:%d  Len:%d",code,contentLen);
		if(code==200){
			read_len = esp_http_client_read(client, content, contentLen);
			if(read_len>0){
				(*time) = strtoul(content,NULL,10);
				ESP_LOGI(TAG,"Time:%s  T:%ld",content,(*time));
			}
			esp_http_client_close(client);
			esp_http_client_cleanup(client);
			return 1;
		}
	}
	esp_http_client_close(client);
    esp_http_client_cleanup(client);
	return 0;	
}



/********设备注册		设备调试信息提交		**********/
char regEspDev(void){	
	saveMqttInfo("18.177.147.131",1883,espInfo.strdevid,"12345678");
	return 1;
	// char tmp = 0;
	// char url[UrlMaxLen];
	// int code = 0;
	// char content[MAX_HTTP_RECV_BUFFER];
	// int contentLen = 0;
	// int read_len = 0;
	
	// memset(url,0,UrlMaxLen);
	// memset(content,0,MAX_HTTP_RECV_BUFFER);
	
	// cJSON *root = cJSON_CreateObject();
    // cJSON_AddStringToObject(root, "RMac", espInfo.strdevid);
	// cJSON_AddStringToObject(root, "SMac", espInfo.strdevid);
	// cJSON_AddStringToObject(root, "Type", DevType);
	// cJSON_AddStringToObject(root, "FW", SoftVer);
	// cJSON_AddStringToObject(root, "HV", DevVer);
	// cJSON_AddStringToObject(root, "PT", espInfo.productTime);
	// cJSON_AddStringToObject(root, "PU", espInfo.productUser);
	// cJSON_AddStringToObject(root, "PK", espInfo.productKey);
	// cJSON_AddStringToObject(root, "FirstReg", "");
	// cJSON_AddStringToObject(root, "LastReg", "");
	
	// char *jbuf = cJSON_PrintUnformatted(root);
    // cJSON_Delete(root);
	
	// if(jbuf==NULL){
	// 	ESP_LOGI(TAG,"regEspDev buf null");
	// 	return 0;
	// }
	
	// sprintf(url,"%s/dev/reg",DevServer);
	
	// ESP_LOGI(TAG,"reg dev:%s  \nContent:%s",url,jbuf);
	
	// esp_http_client_config_t config = {
    //     .url = "http://xapi.elletechnology.com/DevPublicService/dev/reg",
	// 	.event_handler = _http_event_handler,
	// 	.timeout_ms = 20000,
    // };
	
    // esp_http_client_handle_t client = esp_http_client_init(&config);
	
    // esp_http_client_set_method(client, HTTP_METHOD_GET);
	// esp_http_client_set_url(client, url);
	// esp_http_client_set_header(client,"reg",jbuf);

	// esp_err_t err = esp_http_client_perform(client);
    // if (err == ESP_OK) {
	// 	contentLen = esp_http_client_get_content_length(client);
	// 	code = esp_http_client_get_status_code(client);
	// 	ESP_LOGI(TAG,"Code:%d  Len:%d",code,contentLen);
	// 	if(code==200){
			
	// 		free(jbuf);
			
	// 		read_len = esp_http_client_read(client, content, contentLen);
	// 		if(read_len>0){
	// 			ESP_LOGI(TAG,"delet dev return:%s  ",content);
				
	// 			cJSON *rjson = cJSON_Parse(content);
	// 			if(rjson!=NULL){
					
	// 				cJSON *ret = cJSON_GetObjectItem(rjson,"ret");
					
					
	// 				if(ret!=NULL){
						
	// 					if(strcmp(ret->valuestring,"succeed")==0){
							
	// 						cJSON *CIp = cJSON_GetObjectItem(rjson,"CIp");
	// 						cJSON *CPort = cJSON_GetObjectItem(rjson,"CPort");
							
	// 						char mqttKey[32];
	// 						memset(mqttKey,0,32);
	// 						sprintf(mqttKey,"%s-dxswjwo",espInfo.strdevid);
	// 						saveMqttInfo(CIp->valuestring,atoi(CPort->valuestring),espInfo.strdevid,mqttKey);
							
	// 						tmp = 1;
							
	// 					}else{
	// 						cJSON *reason = cJSON_GetObjectItem(rjson,"reason");
							
	// 						ESP_LOGI(TAG,"reg dev failed:%s",reason->valuestring);
	// 					}
						
	// 				}
					
	// 				cJSON_Delete(rjson);
	// 			}
				
	// 		}

	// 		esp_http_client_close(client);
	// 		esp_http_client_cleanup(client);
	// 		return tmp;
	// 	}else{
	// 		ESP_LOGI(TAG,"注册失败:%d",code);
	// 	}
	// }else{
	// 	ESP_LOGI(TAG,"注册请求失败");
	// }
	// free(jbuf);
	// esp_http_client_close(client);
    // esp_http_client_cleanup(client);
	// return 0;
}

char logEsp(char *devid,char *devtype,char *ver,char *msgtype,char *content)
{
	return 1;
}

















