#include "esp_uart.h"


#include "string.h"
#include <ctype.h>
#include <errno.h>
#include <stdlib.h>
#include <time.h>
#include "nvs.h"
#include "nvs_flash.h"
#include "esp_partition.h"
#include "esp_log.h"
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/queue.h"
#include "nvs_flash.h"
#include "driver/uart.h"
#include "driver/gpio.h"

#include "esp_uart.h"
#include "esp_config.h"
#include "cJSON.h"

//{"cmd":"WriteProductInfo","value":{"time":"2020-04-23 09:46:15","user":"admin1","key":"39862157"}}

void initUart(void);
void logHex(char *tag,unsigned char *buf,unsigned int len);
void writeUartBuf(unsigned char *buf,int len);

unsigned long bytesToUL(unsigned char *buf,unsigned char len);
void ulToBytes(unsigned long tmp,unsigned char *buf,unsigned char len);

#define UartDebug		0
#define UartDriveIndex	UART_NUM_0

#define TAG				"EUart"

extern ESPINFO espInfo;

//{"cmd":"DevReset"}
char dealUartJsonCmd(char *buf){
	char tmp = 0;
	if(buf[0]=='{'){
		cJSON * root = cJSON_Parse(buf);
		if(root!=NULL){			
			cJSON *cmd = cJSON_GetObjectItem(root, "cmd");
			if(cmd!=NULL){
				if(strcmp(cmd->valuestring,"WriteProductInfo")==0){
					cJSON *value = cJSON_GetObjectItem(root, "value");
					if(value!=NULL){
						addProductInfo(cJSON_PrintUnformatted(value));
						tmp =1;
						char *result = "{\"cmd\":\"WriteProductInfoOk\"}\0";
						writeUartBuf((unsigned char *)result,strlen(result));
					}
				}
				if(strcmp(cmd->valuestring,"DevRestart")==0){
					char *result = "{\"cmd\":\"DevRestartOk\"}\0";
					writeUartBuf((unsigned char *)result,strlen(result));
					vTaskDelay(500 / portTICK_RATE_MS);
					esp_restart();
				}
				if(strcmp(cmd->valuestring,"DevReset")==0){
					char *result = "{\"cmd\":\"DevResetOk\"}\0";
					writeUartBuf((unsigned char *)result,strlen(result));
					clearEspConfig();
					vTaskDelay(500 / portTICK_RATE_MS);
					esp_restart();
				}
			}
			cJSON_Delete(root);
			return tmp;
		}
		
	}
	return 0;
}

char dealUartHexCmd(unsigned char *buf,unsigned int len){
	return 0;
}

void initEspUart(void){
	initUart();
}


#define BUF_SIZE 			256
void writeUartBuf(unsigned char *buf,int len){
	logHex("US:",buf,len);
	uart_write_bytes(UartDriveIndex, (const char *) buf, len);
}

void sendUartData(unsigned char *buf,unsigned int len){
	writeUartBuf(buf,len);
}

unsigned char uartRecvData[BUF_SIZE];
static void uart_event_task(void *pvParameters){
	unsigned char isTimeSet = 0;
	ESP_LOGI(TAG,"uart0 start ok");
	unsigned int x = 0;
    while (1) {		
		memset(uartRecvData,0,BUF_SIZE);
        int len = uart_read_bytes(UartDriveIndex, uartRecvData, BUF_SIZE, 30 / portTICK_RATE_MS);
		if(len>0){
			ESP_LOGI(TAG,"uart recv len:%d",len);			
			if(dealUartJsonCmd((char *)uartRecvData)>0){			
			}else if(dealUartHexCmd(uartRecvData,len)>0){		
			}else{			
			}			
			//logHex("UR",uartRecvData,len);
			//checkPacket((unsigned char *)uartRecvData,len);
		}		
    }
}
#define TXD_PIN 		GPIO_NUM_1
#define RXD_PIN 		GPIO_NUM_3
void initUart(void){
	uart_config_t uart_config = {
        .baud_rate = 115200,
        .data_bits = UART_DATA_8_BITS,
        .parity    = UART_PARITY_DISABLE,
        .stop_bits = UART_STOP_BITS_1,
        .flow_ctrl = UART_HW_FLOWCTRL_DISABLE
    };
	uart_param_config(UartDriveIndex, &uart_config);
	uart_set_pin(UartDriveIndex, TXD_PIN, RXD_PIN, UART_PIN_NO_CHANGE, UART_PIN_NO_CHANGE);
	uart_driver_install(UartDriveIndex, BUF_SIZE*2,  BUF_SIZE, 10, NULL,0);
	xTaskCreate(uart_event_task, "uart_event_task", 4096, NULL, 0, NULL);
}
void logHex(char *tag,unsigned char *buf,unsigned int len){	
	char msg[512];
	memset(msg,0,512);
	sprintf(msg,"TAG: %s --->  ",tag);
	for(int i=0;i<len;i++){
		int xlen = strlen(msg);
		sprintf(&msg[xlen],"%02X ",buf[i]);
	}	
	ESP_LOGI(TAG,"%s",msg);	
}
unsigned long bytesToUL(unsigned char *buf,unsigned char len){
	unsigned long tmp = 0;
	for(int i=0;i<len;i++){
		tmp |= buf[i];
		if(i!=(len-1)){
			tmp = tmp<<8;
		}
	}	
	return tmp;
}
void ulToBytes(unsigned long tmp,unsigned char *buf,unsigned char len){
	unsigned long xtmp = 0;
	for(int i=0;i<len;i++){
		xtmp = (tmp>>(i*8));
		buf[(len-1)] = xtmp&0xff;
	}
}
unsigned int twoCharToUint(unsigned char *buf){
	unsigned int x = 0;
	x = buf[0];
	x = x<<8;
	x|=buf[1];
	return x;
}























