#include <stdio.h>
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "esp_system.h"
#include "esp_spi_flash.h"
#include "esp_log.h"
#include "nvs_flash.h"

#include "string.h"

#include "elle_iot.h"
#include "esp_ble.h"
#include "esp_uart.h"
#include "esp_new_wifi.h"


#define TAG 				"Main"

static void elleIotParamCbs(unsigned int index,void *value)
{
	ESP_LOGI(TAG,"app set Param :%d ",index);
			
	switch(index){
		case Index_Fun_OnOff:
		{
			unsigned char onoff = (unsigned char)(*((unsigned char *)value));
			if(onoff!=0){
				ESP_LOGI(TAG,"开机");
			}else{
				ESP_LOGI(TAG,"关机");
			}
		}
			break;
		//温度标识		0华氏度		1摄氏度
		case Index_C_F_Flag:
			break;
		//报警时间
		case Index_Alarm_Time:
			break;
		//profile的当前工作步骤
		case Index_Cur_Step:
			break;
		//profile的所有步骤
		case Index_All_Step:
			break;
		//profile的名字
		case Index_Profile_Name:
			break;
		//当前炉子显示温度
		case Index_Grill_Temp:
			break;
		//当前肉探针1显示温度
		case Index_Grill_Probe1:
			break;
		//当前肉探针2显示温度
		case Index_Grill_Probe2:
			break;
		//当前肉探针3显示温度
		case Index_Grill_Probe3:
			break;
		//设置当前炉子温度
		case Index_Grill_Set_Temp:
		{
			unsigned char workmode = 0;	
			elleIOTParam(0,Index_Fun_WorkMode,&workmode);
		}
			break;
		case Index_Fun_PresetSmoke:
		{
			unsigned char workmode = 1;	
			elleIOTParam(0,Index_Fun_WorkMode,&workmode);
		}
		break;
		case Index_Fun_PresetRoast:
		{
			unsigned char workmode = 2;	
			elleIOTParam(0,Index_Fun_WorkMode,&workmode);
		}		
		break;
		case Index_Fun_PresetGrill:
		{
			unsigned char workmode = 3;	
			elleIOTParam(0,Index_Fun_WorkMode,&workmode);
		}
		break;
		case Index_Fun_Prime:
		{
			unsigned char primeMode = (unsigned char)(*((unsigned char *)value));
			elleIOTParam(0,Index_Fun_Prime,&primeMode);
		}
		break;
		case Index_Fun_ProSmoke:
		{
			unsigned char proSmokeMode = (unsigned char)(*((unsigned char *)value));	
			elleIOTParam(0,Index_Fun_ProSmoke,&proSmokeMode);
		}
		break;
		//设置肉探针1温度
		case Index_Grill_Set_Probe1:
			break;
		//设置肉探针2温度
		case Index_Grill_Set_Probe2:
			break;
		//设置肉探针3温度
		case Index_Grill_Set_Probe3:
			break;
		//设置Preset Grill Temp
		case Index_Preset_Grill_Temp:	
			break;
		//设置Preset Smoke Temp
		case Index_Preset_Smoke_Temp:
			break;
		//设置Preset Roast Temp
		case Index_Preset_Roast_Temp:
			break;
		case Index_Fun_Profile_Index:
			break;
		//设置了运行时间
		case Index_Fun_Run_Time:
			break;	
		default:
			break;
	}
}




void app_main()
{	
	printf("SDK version:%s\n", esp_get_idf_version());
    esp_err_t ret;
    ret = nvs_flash_init();
    if (ret == ESP_ERR_NVS_NO_FREE_PAGES) {
        ESP_ERROR_CHECK(nvs_flash_erase());
        ret = nvs_flash_init();
    }
	//初始化互联网库
	initElleIOT();
	//初始化串口
	initEspUart();
	//初始化蓝牙
	initEspBle(espInfo.bdevid[4],espInfo.bdevid[5]);
	//初始化WiFi
	initEspNewWiFi();
	//初始化服务
	if(getProductKeyOkFlag()){		
		ESP_LOGI(TAG,"设备填入了秘钥");		
		//设置app的参数回调 - app 设置参数后，会回调到这里
		setAppParamCbs(elleIotParamCbs);				
		//获取qr码信息
		char qrStr[64];
		memset(qrStr,0,64);
		elleIOTParam(1,Index_QR,qrStr);
		ESP_LOGI(TAG,"QR STR:%s",qrStr);
		//设置工作模式	0手动设置温度 1prime 2prosmoke 3smoke 4roast 5grill
		unsigned char workmode = 1;	
		elleIOTParam(0,Index_Fun_WorkMode,&workmode);
		//默认设置可以被添加，实际使用需要用户切换
		setDeviceCanBeAdd(1);		
		unsigned short grillSetTemp = 300;
		//写入 烧烤炉 主温度
		elleIOTParam(0,Index_Grill_Set_Temp,&grillSetTemp);
		grillSetTemp = 0;
		//读取 烧烤炉 主温度
		elleIOTParam(1,Index_Grill_Set_Temp,&grillSetTemp);
		ESP_LOGI(TAG,"Grill Set Temp:%d",grillSetTemp);		
	}else{
		ESP_LOGI(TAG,"设备还未注册，请填入秘钥，否则无法联网");
	}
	
	//退出进程
	vTaskDelete( NULL );
}















